@@ -236,9 +236,12 @@ class Phonopy(object):
     def get_nac_params(self):
         return self._nac_params
 
-    def get_dynamical_matrix(self):
+    @property
+    def dynamical_matrix(self):
         return self._dynamical_matrix
-    dynamical_matrix = property(get_dynamical_matrix)
+
+    def get_dynamical_matrix(self):
+        return self.dynamical_matrix
 
     def set_unitcell(self, unitcell):
         self._unitcell = unitcell
@@ -578,6 +581,13 @@ class Phonopy(object):
                     self._mesh.get_ir_grid_points(),
                     self._mesh.get_grid_mapping_table())
 
+    @property
+    def mesh_numbers(self):
+        if self._mesh is None:
+            return None
+        else:
+            self._mesh.get_mesh_numbers()
+
     def write_hdf5_mesh(self):
         self._mesh.write_hdf5()
 
@@ -858,8 +868,10 @@ class Phonopy(object):
 
         return plt
 
-    def write_yaml_thermal_properties(self, filename='thermal_properties.yaml'):
-        self._thermal_properties.write_yaml(filename=filename)
+    def write_yaml_thermal_properties(self,
+                                      filename='thermal_properties.yaml'):
+        volume = self.primitive.get_volume()
+        self._thermal_properties.write_yaml(filename=filename, volume=volume)
 
     # Thermal displacement
     def set_thermal_displacements(self,
@@ -872,14 +884,18 @@ class Phonopy(object):
                                   freq_max=None):
         """Prepare thermal displacements calculation
 
-        Args:
-            direction:
-                Projection direction in reduced coordinates.
-            freq_min: Phonons having frequency larger than this are included.
-            freq_max: Phonons having frequency smaller than this are included.
+        Parameters
+        ----------
+        direction: array_like or None
+            Projection direction in reduced coordinates.
+            dtype=float
+            shape=(3,)
+        freq_min: float
+            Phonons having frequency larger than this are included.
+        freq_max: float
+            Phonons having frequency smaller than this are included.
 
         """
-        self._thermal_displacements = None
 
         if self._mesh is not None:
             eigvecs = self._mesh.get_eigenvectors()
@@ -901,12 +917,14 @@ class Phonopy(object):
                 return False
 
         if direction is not None:
-            projection_direction = np.dot(direction, self._primitive.get_cell())
-            td = ThermalDisplacements(iter_phonons,
-                                      self._primitive.get_masses(),
-                                      projection_direction=projection_direction,
-                                      freq_min=freq_min,
-                                      freq_max=freq_max)
+            projection_direction = np.dot(direction,
+                                          self._primitive.get_cell())
+            td = ThermalDisplacements(
+                iter_phonons,
+                self._primitive.get_masses(),
+                projection_direction=projection_direction,
+                freq_min=freq_min,
+                freq_max=freq_max)
         else:
             td = ThermalDisplacements(iter_phonons,
                                       self._primitive.get_masses(),
